#include "Arduino.h"
#include "MiRobot.h"

/* MiRobot */

// MiRobot constructor asigna los parametros (pins de Arduino) a las funciones.

//////////////////   INICIO CONSTRUCTOR   ///////////////////////////////////////

MiRobot::MiRobot(int IB1, int IA1, int IB2, int IA2, int disparoPin, int ecoPin, int infraD, int infraI){
  a1 = IB1;
  a2 = IA1;
  b1 = IB2;
  b2 = IA2;
  disparo = disparoPin;
  eco = ecoPin;
  infrarrojoDerecho = infraD;
  infrarrojoIzquierdo = infraI;

  // Booleano para saber si estan en movimiento los motores.

  avanzaA = false;
  avanzaB = false;

}
// Esta función configura los pines como pines de salida
// en función de Arduino setup()

//////////////////   MÉTODO SETUP   ///////////////////////////////////////

void MiRobot::begin(){
  pinMode(a1, OUTPUT);
  pinMode(a2, OUTPUT);
  pinMode(b1, OUTPUT);
  pinMode(b2, OUTPUT);
  pinMode(disparo, OUTPUT);
  pinMode(eco, INPUT);
}

// Estas son las funciones de movimiento del robot.
// Los parámetos son enteros de 0 a 255.
// Si es verdad que hasta 100/120 no tendremos el voltaje
// suficiente para mover los motores
// Esta llaman a la función motores

//////////////////   FUNCIONES MÉTODOS L9110S  ///////////////////////////////////////

void MiRobot::avanzar(int velocidadA, int velocidadB){
  motores(velocidadA, velocidadB);
}

void MiRobot::retroceder(int velocidadA, int velocidadB){
  motores((-1 * velocidadA), (-1 * velocidadB));
}

// The functions below peform a point turn, at the given velocidadA (Integer between 0 and 255). Arc turns can be initiated by calling left() and right() individually.
void MiRobot::girarIzquierda(int velocidadA, int velocidadB){
  motores((-1 * velocidadA), velocidadB);
}

void MiRobot::girarDerecha(int velocidadA, int velocidadB){
  motores(velocidadA, (-1 * velocidadB));
}

void MiRobot::stop(){

  digitalWrite(a1, HIGH);
  digitalWrite(a2, HIGH);

  digitalWrite(b1, HIGH);
  digitalWrite(b2, HIGH);

  // Guardamos si se mueve el motor
  motorParadoA = 0;
  motorParadoB = 0;
}

// Función que controla los motores individualmente,
// Comprobamos si hay un cambio de dirección.
// Si lo hay paramos el motor
// y realizamos una (acelaración/deceleración) suave.

void MiRobot::motores(int nuevaVelocidadA, int nuevaVelocidadB){
  // Tomamos los valores como "Absolutos" (sin negativos)-
  int movimientoA = abs(nuevaVelocidadA);
  int movimientoB = abs(nuevaVelocidadB);

  // Si se produce un cambio de dirección, apagamos el motor
  if((nuevaVelocidadA > 0) != avanzaA){
    digitalWrite(a1, LOW);
    digitalWrite(a2, LOW);
  // Guardamos el estado de parado
    motorParadoA = 0;
  }

  if((nuevaVelocidadB > 0) != avanzaB){
    digitalWrite(b1, LOW);
    digitalWrite(b2, LOW);
    motorParadoB = 0;
  }

  // Almacenamos si el motor avanza (true) o retrocede (false)
  avanzaA = (nuevaVelocidadA > 0);
  avanzaB = (nuevaVelocidadB > 0);

  // Cuando avanza el pin (a1) el estado es HIGH
  // Si no retrocede y el estado pasa a LOW
  digitalWrite(a1, (avanzaA ? HIGH : LOW));
  digitalWrite(b1, (avanzaB ? HIGH : LOW));

  // Aceleracion / desaceleración
  // Suavizamos los cambios de dirección.
  // Evitamos así forzar los motores.
  // Mientras (el valor motorParado sea distinto de movimientoA)
  // Si el valor motorParado no coincide con movimientoA
  // El valor de motorParadoA aumenta/disminulle
  // motorParado es mayor a movimientoA?
  while (motorParadoA != movimientoA || motorParadoB != movimientoB){

  // Si el valor motorParado no coincide con movimientoA
    if(motorParadoA != movimientoA){

  // El valor de motorParadoA aumenta/disminulle
  // si motorParado es mayor/menor a movimientoA?
      motorParadoA += (motorParadoA > movimientoA ? -1 : 1);

  // Escribe en el pin b2 si avanza (255 - valor motorParado)
  // Si retrocede (255 + valor motorParado)
      analogWrite(a2, (avanzaA ? 255 - motorParadoA : motorParadoA));
    }
    if(motorParadoB != movimientoB){
      motorParadoB += (motorParadoB > movimientoB? -1 : 1);
  // Escribe en el pin b2 si avanza (255 - valor motorParado)
      analogWrite(b2, (avanzaB ? 255 - motorParadoB : motorParadoB));
    }
  // Duración de la aceleración
    delay(1);
  }
  analogWrite(a2, (avanzaA ? 255 - motorParadoA : motorParadoA));
  analogWrite(b2, (avanzaB ? 255 - motorParadoB : motorParadoB));
}

//////////////////   FUNCIONES MÉTODO ULTRASONIDOS   ///////////////////////////////////////

int  MiRobot::ping(int disparo, int eco) {

  //long distanceCm =0;

   long duration, distanceCm;

   digitalWrite(disparo, LOW);  //para generar un pulso limpio ponemos a LOW 4us
   delayMicroseconds(4);
   digitalWrite(disparo, HIGH);  //generamos Trigger (disparo) de 10us
   delayMicroseconds(10);
   digitalWrite(disparo, LOW);

   duration = pulseIn(eco, HIGH);  //medimos el tiempo entre pulsos, en microsegundos

   distanceCm = duration * 10 / 292/ 2;   //convertimos a distancia, en cm

   return distanceCm;
}
bool MiRobot::obstaculoDetectado(int distanciaMinima){

  int cm = ping(disparo, eco);

  if ( cm < distanciaMinima){
    return true;
  }
  else {
    return false;
  }
}

//////////////////   FUNCION MÉTODO INFRARROJOS   ///////////////////////////////////////

int MiRobot::readInfrarrojoDerecho (int infrarrojoDerecho) {

  int valorDerecho;
  valorDerecho = digitalRead(infrarrojoDerecho);
  return valorDerecho;
  delay(1);
}
int MiRobot::readInfrarrojoIzquierdo (int infrarrojoIzquierdo) {

  int valorIzquierdo;
  valorIzquierdo = digitalRead(infrarrojoIzquierdo);
  return valorIzquierdo;
}
